==> trilinos_12.4.2.orig.tar.bz2: trilinos-12.4.2-Source/packages/tpetra/core/src/Tpetra_Details_OrdinalTraits.hpp <==
/*
// @HEADER
// ***********************************************************************
//
//          Tpetra: Templated Linear Algebra Services Package
//                 Copyright (2008) Sandia Corporation
//
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Michael A. Heroux (maherou@sandia.gov)
//
// ************************************************************************
// @HEADER
*/

#ifndef TPETRA_DETAILS_ORDINALTRAITS_HPP
#define TPETRA_DETAILS_ORDINALTRAITS_HPP

/// \file Tpetra_Details_OrdinalTraits.hpp
/// \brief Traits class for "invalid" (flag) values of integer types
///   that Tpetra uses as local ordinals or global ordinals.

#include "Tpetra_ConfigDefs.hpp"
#include <climits>

namespace Tpetra {
namespace Details {

/// \brief Traits class for "invalid" (flag) values of integer types
///   that Tpetra uses as local ordinals or global ordinals.
///
/// \tparam T Built-in integer type.
///
/// I didn't choose these values.  For backwards compatibility, they
/// are the same as the values found in
/// Teuchos::OrdinalTraits<T>::invalid().  I can't call
/// Teuchos::OrdinalTraits<T>::invalid() because it is not marked as a
/// Kokkos device function.  I also can't use std::numeric_limits for
/// the same reason.
template<class T>
struct OrdinalTraits {
  static KOKKOS_INLINE_FUNCTION T invalid () { return -1; }
};

// template<>
// struct OrdinalTraits<char> {
//   static KOKKOS_INLINE_FUNCTION char invalid () { return CHAR_MAX; }
// };

template<>
struct OrdinalTraits<short int> {
  static KOKKOS_INLINE_FUNCTION short int invalid () { return -1; }
};

template<>
struct OrdinalTraits<unsigned short int> {
  static KOKKOS_INLINE_FUNCTION unsigned short int invalid () { return USHRT_MAX; }
};

template<>
struct OrdinalTraits<int> {
  static KOKKOS_INLINE_FUNCTION int invalid () { return -1; }
};

template<>
struct OrdinalTraits<unsigned int> {
  static KOKKOS_INLINE_FUNCTION unsigned int invalid () { return UINT_MAX; }
};

template<>
struct OrdinalTraits<long> {
  static KOKKOS_INLINE_FUNCTION long invalid () { return -1; }
};

template<>
struct OrdinalTraits<unsigned long> {
  static KOKKOS_INLINE_FUNCTION unsigned long invalid () { return ULONG_MAX; }
};

template<>
struct OrdinalTraits<long long> {
  static KOKKOS_INLINE_FUNCTION long long invalid () { return -1; }
};

template<>
struct OrdinalTraits<unsigned long long> {
  static KOKKOS_INLINE_FUNCTION unsigned long long invalid () { return ULLONG_MAX; }
};

} // namespace Details
} // namespace Tpetra

#endif // TPETRA_DETAILS_ORDINALTRAITS_HPP

==> trilinos_12.12.1.orig.tar.bz2: trilinos-12.12.1-Source/packages/tpetra/core/src/Tpetra_Details_OrdinalTraits.hpp <==
/*
// @HEADER
// ***********************************************************************
//
//          Tpetra: Templated Linear Algebra Services Package
//                 Copyright (2008) Sandia Corporation
//
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Michael A. Heroux (maherou@sandia.gov)
//
// ************************************************************************
// @HEADER
*/

#ifndef TPETRA_DETAILS_ORDINALTRAITS_HPP
#define TPETRA_DETAILS_ORDINALTRAITS_HPP

/// \file Tpetra_Details_OrdinalTraits.hpp
/// \brief Import KokkosSparse::OrdinalTraits, a traits class for
///   "invalid" (flag) values of integer types, into the
///   Tpetra::Details namespace.

#include "Tpetra_ConfigDefs.hpp"
#include "Kokkos_Sparse_OrdinalTraits.hpp"

namespace Tpetra {
namespace Details {

// KokkosSparse::OrdinalTraits is a traits class for "invalid" (flag)
// values of integer types.  Tpetra uses those flag values with both
// local and global indices.
using ::KokkosSparse::OrdinalTraits;

} // namespace Details
} // namespace Tpetra

#endif // TPETRA_DETAILS_ORDINALTRAITS_HPP
